@@ -1,13 +1,11 @@
 #!/usr/bin/env python2
-
-'''
-You should normally never use this! Use emcc instead.
+"""You should normally never use this! Use emcc instead.
 
 This is a small wrapper script around the core JS compiler. This calls that
 compiler with the settings given to it. It can also read data from C/C++
 header files (so that the JS compiler can see the constants in those
 headers, for the libc implementation in JS).
-'''
+"""
 
 from tools.toolchain_profiler import ToolchainProfiler
 if __name__ == '__main__':
@@ -16,6 +14,7 @@ if __name__ == '__main__':
 import difflib
 import os, sys, json, argparse, subprocess, re, time, logging
 import shutil
+import pprint
 from collections import OrderedDict
 
 from tools import shared
@@ -61,8 +60,8 @@ def access_quoter(settings):
   return access_quote
 
 
-def emscript(infile, settings, outfile, libraries=None, compiler_engine=None,
-             temp_files=None, DEBUG=None):
+def emscript(infile, settings, outfile, libraries, compiler_engine, temp_files,
+             DEBUG):
   """Runs the emscripten LLVM-to-JS compiler.
 
   Args:
@@ -72,8 +71,6 @@ def emscript(infile, settings, outfile, libraries=None, compiler_engine=None,
     outfile: The file where the output is written.
   """
 
-  if libraries is None: libraries = []
-
   assert settings['ASM_JS'], 'fastcomp is asm.js-only (mode 1 or 2)'
 
   success = False
@@ -147,7 +144,7 @@ def parse_backend_output(backend_output, DEBUG):
   mem_init = mem_init.replace('Runtime.', '')
 
   try:
-    #if DEBUG: print >> sys.stderr, "METAraw", metadata_raw
+    #if DEBUG: logging.debug("METAraw %s", metadata_raw)
     metadata = json.loads(metadata_raw, object_pairs_hook=OrderedDict)
   except Exception as e:
     logging.error('emscript: failure to parse metadata output from compiler backend. raw output is: \n' + metadata_raw)
@@ -161,7 +158,9 @@ def parse_backend_output(backend_output, DEBUG):
 
 def fixup_metadata_tables(metadata, settings):
   # if emulating pointer casts, force all tables to the size of the largest
-  if settings['EMULATE_FUNCTION_POINTER_CASTS']:
+  # (for wasm, we use binaryen's fpcast-emu pass, we don't need to do anything
+  # here)
+  if settings['EMULATE_FUNCTION_POINTER_CASTS'] and not settings['WASM']:
     max_size = 0
     for k, v in metadata['tables'].items():
       max_size = max(max_size, v.count(',')+1)
@@ -325,7 +324,7 @@ def function_tables_and_exports(funcs, metadata, mem_init, glue, forwarded_data,
 
   function_tables_impls = make_function_tables_impls(function_table_data, settings)
   final_function_tables = '\n'.join(function_tables_impls) + '\n' + function_tables_defs
-  if settings.get('EMULATED_FUNCTION_POINTERS'):
+  if settings['EMULATED_FUNCTION_POINTERS']:
     final_function_tables = (
       final_function_tables
       .replace("asm['", '')
@@ -375,7 +374,7 @@ def create_module(function_table_sigs, metadata, settings,
   asm_start = asm_start_pre + '\n' + asm_global_vars + asm_temp_vars + asm_runtime_thread_local_vars + '\n' + asm_global_funcs
 
   temp_float = '  var tempFloat = %s;\n' % ('Math_fround(0)' if provide_fround(settings) else '0.0')
-  async_state = '  var asyncState = 0;\n' if settings.get('EMTERPRETIFY_ASYNC') else ''
+  async_state = '  var asyncState = 0;\n' if settings['EMTERPRETIFY_ASYNC'] else ''
   f0_fround = '  const f0 = Math_fround(0);\n' if provide_fround(settings) else ''
 
   replace_memory = create_replace_memory(settings)
@@ -461,7 +460,7 @@ def create_backend_args(infile, temp_js, settings):
     args += ['-emscripten-asyncify-whitelist=' + ','.join(settings['ASYNCIFY_WHITELIST'])]
   if settings['NO_EXIT_RUNTIME']:
     args += ['-emscripten-no-exit-runtime']
-  if settings['BINARYEN']:
+  if settings['WASM']:
     args += ['-emscripten-wasm']
     if shared.Building.is_wasm_only():
       args += ['-emscripten-only-wasm']
@@ -548,7 +547,7 @@ def compile_settings(compiler_engine, settings, libraries, temp_files):
 
 
 def memory_and_global_initializers(pre, metadata, mem_init, settings):
-  global_initializers = str(', '.join(['{ func: function() { %s() } }' % i for i in metadata['initializers']]))
+  global_initializers = ', '.join('{ func: function() { %s() } }' % i for i in metadata['initializers'])
 
   if settings['SIMD'] == 1:
     pre = open(path_from_root(os.path.join('src', 'ecmascript_simd.js'))).read() + '\n\n' + pre
@@ -573,7 +572,7 @@ def memory_and_global_initializers(pre, metadata, mem_init, settings):
 
   if settings['SIDE_MODULE']:
     pre = pre.replace('GLOBAL_BASE', 'gb')
-  if settings['SIDE_MODULE'] or settings['BINARYEN']:
+  if settings['SIDE_MODULE'] or settings['WASM']:
     pre = pre.replace('{{{ STATIC_BUMP }}}', str(staticbump))
 
   return pre
@@ -615,7 +614,7 @@ def get_original_exported_functions(settings):
 
 
 def check_all_implemented(all_implemented, pre, settings):
-  if settings['ASSERTIONS'] and settings.get('ORIGINAL_EXPORTED_FUNCTIONS'):
+  if settings['ASSERTIONS'] and settings['ORIGINAL_EXPORTED_FUNCTIONS']:
     original_exports = get_original_exported_functions(settings)
     for requested in original_exports:
       if not is_already_implemented(requested, pre, all_implemented):
@@ -649,7 +648,7 @@ def get_exported_implemented_functions(all_exported_functions, all_implemented,
       funcs += ['setDynamicTop']
     if not settings['RELOCATABLE']:
       funcs += ['setTempRet0', 'getTempRet0']
-    if not (settings['BINARYEN'] and settings['SIDE_MODULE']):
+    if not (settings['WASM'] and settings['SIDE_MODULE']):
       funcs += ['setThrew']
     if settings['EMTERPRETIFY']:
       funcs += ['emterpret']
@@ -671,13 +670,16 @@ def proxy_debug_print(call_type, settings):
   return ''
 
 def include_asm_consts(pre, forwarded_json, metadata, settings):
-  if settings['BINARYEN'] and settings['SIDE_MODULE']:
+  if settings['WASM'] and settings['SIDE_MODULE']:
     assert len(metadata['asmConsts']) == 0, 'EM_ASM is not yet supported in shared wasm module (it cannot be stored in the wasm itself, need some solution)'
 
   asm_consts, all_sigs, call_types = all_asm_consts(metadata)
   asm_const_funcs = []
   for s in range(len(all_sigs)):
     sig = all_sigs[s]
+    if 'j' in sig:
+      logging.error('emscript: EM_ASM should not receive i64s as inputs, they are not valid in JS')
+      sys.exit(1)
     call_type = call_types[s] if s < len(call_types) else ''
     if '_emscripten_asm_const_' + call_type + sig in forwarded_json['Functions']['libraryFunctions']: continue # Only one invoker needs to be emitted for each ASM_CONST (signature x call_type) item
     forwarded_json['Functions']['libraryFunctions']['_emscripten_asm_const_' + call_type + sig] = 1
@@ -770,17 +772,16 @@ def make_function_tables_defs(implemented_functions, all_implemented, function_t
   def make_coercions(sig): return ';'.join(['p%d = %s' % (p, shared.JS.make_coercion('p%d' % p, sig[p+1], settings)) for p in range(len(sig)-1)]) + ';'
 
   # when emulating function pointer casts, we need to know what is the target of each pointer
-  if settings['EMULATE_FUNCTION_POINTER_CASTS']:
+  if settings['EMULATE_FUNCTION_POINTER_CASTS'] and not settings['WASM']:
     function_pointer_targets = {}
     for sig, table in function_table_data.items():
       start = table.index('[')
       end = table.rindex(']')
       body = table[start+1:end].split(',')
-      parsed = [x.strip() for x in body]
-      for i in range(len(parsed)):
-        if parsed[i] != '0':
+      for i, parsed in enumerate(x.strip() for x in body):
+        if parsed != '0':
           assert i not in function_pointer_targets
-          function_pointer_targets[i] = [sig, str(parsed[i])]
+          function_pointer_targets[i] = [sig, str(parsed)]
 
   def make_table(sig, raw):
     if '[]' in raw: return ('', '') # empty table
@@ -827,7 +828,7 @@ def make_function_tables_defs(implemented_functions, all_implemented, function_t
       Counter.next_item += 1
       newline = Counter.next_item % 30 == 29
       if item == '0':
-        if j > 0 and settings['EMULATE_FUNCTION_POINTER_CASTS'] and j in function_pointer_targets: # emulate all non-null pointer calls, if asked to
+        if j > 0 and settings['EMULATE_FUNCTION_POINTER_CASTS'] and not settings['WASM'] and j in function_pointer_targets: # emulate all non-null pointer calls, if asked to
           proper_sig, proper_target = function_pointer_targets[j]
           if settings['EMULATED_FUNCTION_POINTERS']:
             if proper_target in all_implemented:
@@ -862,7 +863,7 @@ def make_function_tables_defs(implemented_functions, all_implemented, function_t
       # when emulating function pointers, we don't need wrappers
       # but if relocating, then we also have the copies in-module, and do
       # in wasm we never need wrappers though
-      if clean_item not in implemented_functions and not (settings['EMULATED_FUNCTION_POINTERS'] and not settings['RELOCATABLE']) and not settings['BINARYEN']:
+      if clean_item not in implemented_functions and not (settings['EMULATED_FUNCTION_POINTERS'] and not settings['RELOCATABLE']) and not settings['WASM']:
         # this is imported into asm, we must wrap it
         call_ident = clean_item
         if call_ident in metadata['redirects']: call_ident = metadata['redirects'][call_ident]
@@ -935,8 +936,8 @@ function jsCall_%s_%s(%s) {
 
 def create_mftCall_funcs(function_table_data, settings):
   mftCall_funcs = []
-  if settings.get('EMULATED_FUNCTION_POINTERS'):
-    if settings.get('RELOCATABLE') and not settings['BINARYEN']: # in wasm, emulated function pointers are just simple table calls
+  if settings['EMULATED_FUNCTION_POINTERS']:
+    if settings['RELOCATABLE'] and not settings['WASM']: # in wasm, emulated function pointers are just simple table calls
       for sig, table in function_table_data.items():
         params = ','.join(['ptr'] + ['p%d' % p for p in range(len(sig)-1)])
         coerced_params = ','.join([shared.JS.make_coercion('ptr', 'i', settings)] + [shared.JS.make_coercion('p%d', unfloat(sig[p+1]), settings) % p for p in range(len(sig)-1)])
@@ -1010,7 +1011,7 @@ def create_asm_global_funcs(bg_funcs, metadata, settings):
 def create_asm_global_vars(bg_vars, settings):
   access_quote = access_quoter(settings)
   asm_global_vars = ''.join(['  var ' + g + '=env' + access_quote(g) + '|0;\n' for g in bg_vars])
-  if settings['BINARYEN'] and settings['SIDE_MODULE']:
+  if settings['WASM'] and settings['SIDE_MODULE']:
     # wasm side modules internally define their stack, these are set at module startup time
     asm_global_vars += '\n  var STACKTOP = 0, STACK_MAX = 0;\n'
 
@@ -1023,10 +1024,7 @@ def global_simd_funcs(access_quote, metadata, settings):
     return ''
 
   def string_contains_any(s, str_list):
-    for sub in str_list:
-      if sub in s:
-        return True
-    return False
+    return any(sub in s for sub in str_list)
 
   nonexisting_simd_symbols = ['Int8x16_fromInt8x16', 'Uint8x16_fromUint8x16', 'Int16x8_fromInt16x8', 'Uint16x8_fromUint16x8', 'Int32x4_fromInt32x4', 'Uint32x4_fromUint32x4', 'Float32x4_fromFloat32x4', 'Float64x2_fromFloat64x2']
   nonexisting_simd_symbols += ['Int32x4_addSaturate', 'Int32x4_subSaturate', 'Uint32x4_addSaturate', 'Uint32x4_subSaturate']
@@ -1157,7 +1155,7 @@ def create_asm_setup(debug_tables, function_table_data, metadata, settings):
   if settings['ASSERTIONS']:
     for sig in function_table_sigs:
       asm_setup += '\nfunction nullFunc_' + sig + '(x) { ' + get_function_pointer_error(sig, function_table_sigs, settings) + 'abort(x) }\n'
-  if settings['BINARYEN']:
+  if settings['WASM']:
     def table_size(table):
       table_contents = table[table.index('[') + 1: table.index(']')]
       if len(table_contents) == 0: # empty table
@@ -1183,7 +1181,7 @@ def create_asm_setup(debug_tables, function_table_data, metadata, settings):
 
   asm_setup += setup_function_pointers(function_table_sigs, settings)
 
-  if settings.get('EMULATED_FUNCTION_POINTERS'):
+  if settings['EMULATED_FUNCTION_POINTERS']:
     function_tables_impls = make_function_tables_impls(function_table_data, settings)
     asm_setup += '\n' + '\n'.join(function_tables_impls) + '\n'
 
@@ -1194,19 +1192,32 @@ def setup_function_pointers(function_table_sigs, settings):
   asm_setup = ''
   for sig in function_table_sigs:
     asm_setup += '\n' + shared.JS.make_invoke(sig) + '\n'
-    if settings.get('RESERVED_FUNCTION_POINTERS'):
+    if settings['RESERVED_FUNCTION_POINTERS']:
       asm_setup += '\n' + shared.JS.make_jscall(sig) + '\n'
-    if settings.get('EMULATED_FUNCTION_POINTERS'):
+    if settings['EMULATED_FUNCTION_POINTERS']:
       args = ['a%d' % i for i in range(len(sig)-1)]
       full_args = ['x'] + args
-      table_access = 'FUNCTION_TABLE_' + sig
-      if settings['SIDE_MODULE']:
-        table_access = 'parentModule["' + table_access + '"]' # side module tables were merged into the parent, we need to access the global one
-      if settings['BINARYEN']:
-        # wasm uses a Table, which means we have function pointer emulation capabilities all the time, at no cost. just call the table
-        table_access = "Module['wasmTable']"
-        table_read = table_access + '.get(x)'
+      if settings['WASM']:
+        if settings['EMULATE_FUNCTION_POINTER_CASTS']:
+          # emulated function pointers in wasm use an internal i64-based ABI with a fixed number of arguments. we can't
+          # call into it directly because it returns an i64, which is an error for the VM. instead, we use dynCalls
+          dyn_call = "Module['asm']['dynCall_" + sig + "']"
+          asm_setup += '''
+function ftCall_%s(%s) {
+  return %s(%s);
+}
+''' % (sig, ', '.join(full_args), dyn_call, ', '.join(full_args))
+          # and we are done with this signature, continue
+          continue
+        else:
+          # otherwise, wasm emulated function pointers *without* emulated casts can just all
+          # into the table
+          table_access = "Module['wasmTable']"
+          table_read = table_access + '.get(x)'
       else:
+        table_access = 'FUNCTION_TABLE_' + sig
+        if settings['SIDE_MODULE']:
+          table_access = 'parentModule["' + table_access + '"]' # side module tables were merged into the parent, we need to access the global one
         table_read = table_access + '[x]'
       prelude = '''
   if (x < 0 || x >= %s.length) { Module.printErr("Function table mask error (out of range)"); %s ; abort(x) }''' % (table_access, get_function_pointer_error(sig, function_table_sigs, settings))
@@ -1237,25 +1248,20 @@ def create_basic_funcs(function_table_sigs, settings):
 
   for sig in function_table_sigs:
     basic_funcs.append('invoke_%s' % sig)
-    if settings.get('RESERVED_FUNCTION_POINTERS'):
+    if settings['RESERVED_FUNCTION_POINTERS']:
       basic_funcs.append('jsCall_%s' % sig)
-    if settings.get('EMULATED_FUNCTION_POINTERS'):
-      if not settings['BINARYEN']: # in wasm, emulated function pointers are just simple table calls
+    if settings['EMULATED_FUNCTION_POINTERS']:
+      if not settings['WASM']: # in wasm, emulated function pointers are just simple table calls
         basic_funcs.append('ftCall_%s' % sig)
   return basic_funcs
 
 
 def create_basic_vars(exported_implemented_functions, forwarded_json, metadata, settings):
   basic_vars = ['DYNAMICTOP_PTR', 'tempDoublePtr', 'ABORT']
-  if not (settings['BINARYEN'] and settings['SIDE_MODULE']):
+  if not (settings['WASM'] and settings['SIDE_MODULE']):
     basic_vars += ['STACKTOP', 'STACK_MAX']
-  if metadata.get('preciseI64MathUsed'):
-    basic_vars += ['cttz_i8']
-  else:
-    if forwarded_json['Functions']['libraryFunctions'].get('_llvm_cttz_i32'):
-      basic_vars += ['cttz_i8']
   if settings['RELOCATABLE']:
-    if not (settings['BINARYEN'] and settings['SIDE_MODULE']):
+    if not (settings['WASM'] and settings['SIDE_MODULE']):
       basic_vars += ['gb', 'fb']
     else:
       basic_vars += ['memoryBase', 'tableBase'] # wasm side modules have a specific convention for these
@@ -1276,19 +1282,24 @@ def create_exports(exported_implemented_functions, in_table, function_table_data
   exports = []
   for export in sorted(set(all_exported)):
     exports.append(quote(export) + ": " + export)
-  if settings['BINARYEN'] and settings['SIDE_MODULE']:
+  if settings['WASM'] and settings['SIDE_MODULE']:
     # named globals in side wasm modules are exported globals from asm/wasm
     for k, v in metadata['namedGlobals'].items():
       exports.append(quote('_' + str(k)) + ': ' + str(v))
     # aliases become additional exports
     for k, v in metadata['aliases'].items():
       exports.append(quote(str(k)) + ': ' + str(v))
+  # shared wasm emulated function pointer mode requires us to know the function pointer for
+  # each function. export fp$func => function pointer for func
+  if settings['WASM'] and settings['RELOCATABLE'] and settings['EMULATED_FUNCTION_POINTERS']:
+    for k, v in metadata['functionPointers'].items():
+      exports.append(quote('fp$' + str(k)) + ': ' + str(v))
   return '{ ' + ', '.join(exports) + ' }'
 
 
 def create_asm_runtime_funcs(settings):
   funcs = []
-  if not (settings['BINARYEN'] and settings['SIDE_MODULE']):
+  if not (settings['WASM'] and settings['SIDE_MODULE']):
     funcs += ['stackAlloc', 'stackSave', 'stackRestore', 'establishStackSpace', 'setThrew']
   if not settings['RELOCATABLE']:
     funcs += ['setTempRet0', 'getTempRet0']
@@ -1341,15 +1352,15 @@ def create_receiving(function_table_data, function_tables_defs, exported_impleme
     receiving += 'Module["asm"] = asm;\n' + ';\n'.join(['var ' + s + ' = Module["' + s + '"] = function() {' + runtime_assertions + '  return Module["asm"]["' + s + '"].apply(null, arguments) }' for s in exported_implemented_functions + function_tables(function_table_data, settings)])
   receiving += ';\n'
 
-  if settings['EXPORT_FUNCTION_TABLES'] and not settings['BINARYEN']:
+  if settings['EXPORT_FUNCTION_TABLES'] and not settings['WASM']:
     for table in function_table_data.values():
       tableName = table.split()[1]
       table = table.replace('var ' + tableName, 'var ' + tableName + ' = Module["' + tableName + '"]')
       receiving += table + '\n'
 
-  if settings.get('EMULATED_FUNCTION_POINTERS'):
+  if settings['EMULATED_FUNCTION_POINTERS']:
     receiving += '\n' + function_tables_defs.replace('// EMSCRIPTEN_END_FUNCS\n', '') + '\n' + ''.join(['Module["dynCall_%s"] = dynCall_%s\n' % (sig, sig) for sig in function_table_data])
-    if not settings['BINARYEN']:
+    if not settings['WASM']:
       for sig in function_table_data.keys():
         name = 'FUNCTION_TABLE_' + sig
         fullname = name if not settings['SIDE_MODULE'] else ('SIDE_' + name)
@@ -1368,7 +1379,7 @@ for (var named in NAMED_GLOBALS) {
 }
 Module['NAMED_GLOBALS'] = NAMED_GLOBALS;
 ''' % ', '.join('"' + k + '": ' + str(v) for k, v in metadata['namedGlobals'].items())
-    if settings['BINARYEN']:
+    if settings['WASM']:
       # wasm side modules are pure wasm, and cannot create their g$..() methods, so we help them out
       # TODO: this works if we are the main module, but if the supplying module is later, it won't, so
       #       we'll need another solution for that. one option is to scan the module imports, if/when
@@ -1376,7 +1387,8 @@ Module['NAMED_GLOBALS'] = NAMED_GLOBALS;
       named_globals += '''
 for (var named in NAMED_GLOBALS) {
   (function(named) {
-    Module['g$_' + named] = function() { return Module['_' + named] };
+    var func = Module['_' + named];
+    Module['g$_' + named] = function() { return func };
   })(named);
 }
 '''
@@ -1722,15 +1734,13 @@ HEAP_TYPE_INFOS = [
 ]
 
 
-def emscript_wasm_backend(infile, settings, outfile, libraries=None, compiler_engine=None,
-                          temp_files=None, DEBUG=None):
+def emscript_wasm_backend(infile, settings, outfile, libraries, compiler_engine,
+                          temp_files, DEBUG):
   # Overview:
   #   * Run LLVM backend to emit .s
   #   * Run Binaryen's s2wasm to generate WebAssembly.
   #   * We may also run some Binaryen passes here.
 
-  if libraries is None: libraries = []
-
   if shared.Settings.EXPERIMENTAL_USE_LLD:
     wasm, metadata = build_wasm_lld(temp_files, infile, outfile, settings, DEBUG)
   else:
@@ -1758,7 +1768,7 @@ def emscript_wasm_backend(infile, settings, outfile, libraries=None, compiler_en
 
   # memory and global initializers
 
-  global_initializers = str(', '.join(['{ func: function() { %s() } }' % i for i in metadata['initializers']]))
+  global_initializers = ', '.join('{ func: function() { %s() } }' % i for i in metadata['initializers'])
 
   staticbump = metadata['staticBump']
   while staticbump % 16 != 0: staticbump += 1
@@ -1824,7 +1834,6 @@ def build_wasm(temp_files, infile, outfile, settings, DEBUG):
       t = time.time()
       shutil.copyfile(temp_s, os.path.join(shared.CANONICAL_TEMP_DIR, 'emcc-llvm-backend-output.s'))
 
-    assert shared.Settings.BINARYEN_ROOT, 'need BINARYEN_ROOT config set so we can use Binaryen s2wasm on the backend output'
     basename = shared.unsuffixed(outfile.name)
     wasm = basename + '.wasm'
     metadata_file = basename + '.metadata'
@@ -1852,10 +1861,9 @@ def build_wasm(temp_files, infile, outfile, settings, DEBUG):
 
 
 def build_wasm_lld(temp_files, infile, outfile, settings, DEBUG):
-  assert shared.Settings.BINARYEN_ROOT, 'need BINARYEN_ROOT config set so we can use Binaryen tools on the backend output'
-  wasm_emscripten_finalize = os.path.join(shared.Settings.BINARYEN_ROOT, 'bin', 'wasm-emscripten-finalize')
-  wasm_as = os.path.join(shared.Settings.BINARYEN_ROOT, 'bin', 'wasm-as')
-  wasm_dis = os.path.join(shared.Settings.BINARYEN_ROOT, 'bin', 'wasm-dis')
+  wasm_emscripten_finalize = os.path.join(shared.BINARYEN_ROOT, 'bin', 'wasm-emscripten-finalize')
+  wasm_as = os.path.join(shared.BINARYEN_ROOT, 'bin', 'wasm-as')
+  wasm_dis = os.path.join(shared.BINARYEN_ROOT, 'bin', 'wasm-dis')
 
   def debug_copy(src, dst):
     if DEBUG:
@@ -1883,8 +1891,8 @@ def build_wasm_lld(temp_files, infile, outfile, settings, DEBUG):
 
     libc_rt_lib = shared.Cache.get('wasm_libc_rt.a', wasm_rt_fail('wasm_libc_rt.a'), 'a')
     compiler_rt_lib = shared.Cache.get('wasm_compiler_rt.a', wasm_rt_fail('wasm_compiler_rt.a'), 'a')
-    cmd = [shared.LLD, '-flavor', 'wasm',
-      '-z', '-stack-size=1048576',
+    cmd = [shared.WASM_LD,
+      '-z', 'stack-size=%s' % settings['TOTAL_STACK'],
       '--global-base=%s' % shared.Settings.GLOBAL_BASE,
       '--initial-memory=%s' % shared.Settings.TOTAL_MEMORY,
       temp_o, libc_rt_lib, compiler_rt_lib,
@@ -1922,9 +1930,9 @@ def read_metadata_wast(wast, DEBUG):
 
 
 def create_metadata_wasm(metadata_raw, DEBUG):
-  if DEBUG: logging.debug("METAraw %s", metadata_raw)
+  if DEBUG: logging.debug("Metadata raw: " + metadata_raw)
   metadata = load_metadata(metadata_raw)
-  if DEBUG: logging.debug(repr(metadata))
+  if DEBUG: logging.debug("Metadata parsed: " + pprint.pformat(metadata))
   return metadata
 
 
@@ -1934,7 +1942,7 @@ def create_exported_implemented_functions_wasm(pre, forwarded_json, metadata, se
   all_exported_functions = set(shared.expand_response(settings['EXPORTED_FUNCTIONS'])) # both asm.js and otherwise
   for additional_export in settings['DEFAULT_LIBRARY_FUNCS_TO_INCLUDE']: # additional functions to export from asm, if they are implemented
     all_exported_functions.add('_' + additional_export)
-  all_implemented = metadata['implementedFunctions'] + list(forwarded_json['Functions']['implementedFunctions'].keys()) # XXX perf?
+  all_implemented = get_all_implemented(forwarded_json, metadata)
 
   export_bindings = settings['EXPORT_BINDINGS']
   export_all = settings['EXPORT_ALL']
@@ -1942,7 +1950,7 @@ def create_exported_implemented_functions_wasm(pre, forwarded_json, metadata, se
     if key in all_exported_functions or export_all or (export_bindings and key.startswith('_emscripten_bind')):
       exported_implemented_functions.add(key)
 
-  if settings['ASSERTIONS'] and settings.get('ORIGINAL_EXPORTED_FUNCTIONS'):
+  if settings['ASSERTIONS'] and settings['ORIGINAL_EXPORTED_FUNCTIONS']:
     original_exports = get_original_exported_functions(settings)
     for requested in original_exports:
       # check if already implemented
@@ -2141,7 +2149,7 @@ def create_s2wasm_args(temp_s, wasm):
   compiler_rt_lib = shared.Cache.get('wasm_compiler_rt.a', wasm_rt_fail('wasm_compiler_rt.a'), 'a')
   libc_rt_lib = shared.Cache.get('wasm_libc_rt.a', wasm_rt_fail('wasm_libc_rt.a'), 'a')
 
-  s2wasm_path = os.path.join(shared.Settings.BINARYEN_ROOT, 'bin', 's2wasm')
+  s2wasm_path = os.path.join(shared.BINARYEN_ROOT, 'bin', 's2wasm')
 
   args = [s2wasm_path, temp_s, '-o', wasm, '--emscripten-glue', '--emit-binary']
   args += ['--global-base=%d' % shared.Settings.GLOBAL_BASE]
@@ -2176,11 +2184,19 @@ def load_metadata(metadata_raw):
     'invokeFuncs': [],
   }
 
-  for k, v in metadata_json.items():
-    metadata[k] = v
+  for key, value in metadata_json.items():
+    # json.loads returns `unicode` for strings but other code in this file
+    # generally works with utf8 encoded `str` objects, and they don't alwasy
+    # mix well.  e.g. s.replace(x, y) will blow up is `s` a uts8 str containing
+    # non-ascii and either x or y are unicode objects.
+    # TODO(sbc): Remove this encoding if we switch to unicode elsewhere
+    # (specifically the glue returned from compile_settings)
+    if type(value) == list:
+      value = [asstr(v) for v in value]
+    metadata[key] = value
 
   # Initializers call the global var version of the export, so they get the mangled name.
-  metadata['initializers'] = list(map(asmjs_mangle, metadata['initializers']))
+  metadata['initializers'] = [asmjs_mangle(i) for i in metadata['initializers']]
 
   # functions marked llvm.used in the code are exports requested by the user
   shared.Building.user_requested_exports += metadata['exports']
@@ -2216,11 +2232,6 @@ def asmjs_mangle(name):
   return '_' + ''.join(['_' if not c.isalnum() else c for c in name])
 
 
-if os.environ.get('EMCC_FAST_COMPILER') == '0':
-  logging.critical('Non-fastcomp compiler is no longer available, please use fastcomp or an older version of emscripten')
-  sys.exit(1)
-
-
 def normalize_line_endings(text):
   """Normalize to UNIX line endings.
 
@@ -2240,15 +2251,18 @@ def main(args, compiler_engine, cache, temp_files, DEBUG):
     settings[name] = asstr(value)
 
   # libraries
-  libraries = args.libraries[0].split(',') if len(args.libraries) > 0 else []
+  libraries = args.libraries[0].split(',') if len(args.libraries) else []
 
-  settings.setdefault('STRUCT_INFO', shared.path_from_root('src', 'struct_info.compiled.json'))
-  struct_info = settings.get('STRUCT_INFO')
-
-  if not os.path.exists(struct_info) and not settings.get('BOOTSTRAPPING_STRUCT_INFO') and not settings.get('ONLY_MY_CODE'):
-    if DEBUG: logging.debug('  emscript: bootstrapping struct info to %s', struct_info)
-    shared.Building.ensure_struct_info(struct_info)
-    if DEBUG: logging.debug('  emscript: bootstrapping struct info complete')
+  if not settings['BOOTSTRAPPING_STRUCT_INFO'] and not settings['ONLY_MY_CODE']:
+    generated_struct_info_name = 'generated_struct_info.json'
+    def ensure_struct_info():
+      with ToolchainProfiler.profile_block('gen_struct_info'):
+        from tools import gen_struct_info
+        out = shared.Cache.get_path(generated_struct_info_name)
+        gen_struct_info.main(['-qo', out, path_from_root('src', 'struct_info.json')])
+        return out
+    settings['STRUCT_INFO'] = shared.Cache.get(generated_struct_info_name, ensure_struct_info, extension='json')
+  # do we need an else, to define it for the bootstrap case?
 
   emscripter = emscript_wasm_backend if settings['WASM_BACKEND'] else emscript
 
@@ -2256,17 +2270,13 @@ def main(args, compiler_engine, cache, temp_files, DEBUG):
              temp_files=temp_files, DEBUG=DEBUG)
 
 
-def _main(args=None):
-  if args is None:
-    args = sys.argv[1:]
-
-  substitute_response_files(args)
+def _main(args):
+  args = substitute_response_files(args)
 
   parser = argparse.ArgumentParser(
     usage='%(prog)s [-h] [-H HEADERS] [-o OUTFILE] [-c COMPILER_ENGINE] [-s FOO=BAR]* infile',
     description=('You should normally never use this! Use emcc instead. '
-                 'This is a wrapper around the JS compiler, converting .ll to .js.'),
-    epilog='')
+                 'This is a wrapper around the JS compiler, converting .ll to .js.'))
   parser.add_argument('-H', '--headers',
                     default=[],
                     action='append',
@@ -2319,7 +2329,9 @@ WARNING: You should normally never use this! Use emcc instead.
   ''')
 
   if len(positional) != 1:
-    raise RuntimeError('Must provide exactly one positional argument. Got ' + str(len(positional)) + ': "' + '", "'.join(positional) + '"')
+    logging.error('Must provide exactly one positional argument. Got ' + str(len(positional)) + ': "' + '", "'.join(positional) + '"')
+    return 1
+
   keywords.infile = os.path.abspath(positional[0])
   if isinstance(keywords.outfile, (type(u''), bytes)):
     keywords.outfile = open(keywords.outfile, 'w')
@@ -2349,7 +2361,7 @@ WARNING: You should normally never use this! Use emcc instead.
     temp_files=temp_files,
     DEBUG=DEBUG,
   ))
+  return 0
 
 if __name__ == '__main__':
-  _main()
-  sys.exit(0)
+  sys.exit(_main(sys.argv[1:]))
