@@ -473,7 +473,7 @@ var LibraryGLFW = {
 
       if (event.target != Module["canvas"]) return;
 
-      eventButton = GLFW.DOMToGLFWMouseButton(event);
+      var eventButton = GLFW.DOMToGLFWMouseButton(event);
 
       if (status == 1) { // GLFW_PRESS
         GLFW.active.buttons |= (1 << eventButton);
@@ -539,7 +539,7 @@ var LibraryGLFW = {
 
       var resizeNeeded = true;
 
-      // If the client is requestiong fullscreen mode
+      // If the client is requesting fullscreen mode
       if (document["fullscreen"] || document["fullScreen"] || document["mozFullScreen"] || document["webkitIsFullScreen"]) {
         GLFW.active.storedX = GLFW.active.x;
         GLFW.active.storedY = GLFW.active.y;
@@ -774,7 +774,7 @@ var LibraryGLFW = {
           }
 
           var data = e.target.result;
-          FS.writeFile(path, new Uint8Array(data), { encoding: 'binary' });
+          FS.writeFile(path, new Uint8Array(data));
           if (++written === count) {
             Module['dynCall_viii'](GLFW.active.dropFunc, GLFW.active.id, count, filenames);
 
@@ -1040,7 +1040,7 @@ var LibraryGLFW = {
     },
 
     GLFW2ParamToGLFW3Param: function(param) {
-      table = {
+      var table = {
         0x00030001:0, // GLFW_MOUSE_CURSOR
         0x00030002:0, // GLFW_STICKY_KEYS
         0x00030003:0, // GLFW_STICKY_MOUSE_BUTTONS
