@@ -173,8 +173,9 @@ var LibraryManager = {
     for (var i = 0; i < libraries.length; i++) {
       var filename = libraries[i];
       var src = read(filename);
+      var processed = undefined;
       try {
-        var processed = processMacros(preprocess(src, filename));
+        processed = processMacros(preprocess(src, filename));
         eval(processed);
       } catch(e) {
         var details = [e, e.lineNumber ? 'line number: ' + e.lineNumber : '', (e.stack || "").toString().replace('Object.<anonymous>', filename)];
@@ -268,7 +269,7 @@ var LibraryManager = {
 // Safe way to access a C define. We check that we don't add library functions with missing defines.
 function cDefine(key) {
 	if (key in C_DEFINES) return C_DEFINES[key];
-	throw 'XXX missing C define ' + key + '!';
+	throw 'Missing C define ' + key + '! If you just added it to struct_info.json, you need to ./emcc --clear-cache';
 }
 
 var EXPORTED_RUNTIME_METHODS_SET = set(EXPORTED_RUNTIME_METHODS.concat(EXTRA_EXPORTED_RUNTIME_METHODS));
@@ -396,6 +397,9 @@ function exportRuntime() {
     'makeBigInt',
     'dynCall',
     'getCompilerSetting',
+    'stackSave',
+    'stackRestore',
+    'stackAlloc',
   ];
   if (SUPPORT_BASE64_EMBEDDING) {
     runtimeElements.push('intArrayFromBase64');
