@@ -87,7 +87,7 @@ namespace emscripten {
             EM_VAL _emval_typeof(EM_VAL value);
         }
 
-        template<const char* address> 
+        template<const char* address>
         struct symbol_registrar {
             symbol_registrar() {
                 internal::_emval_register_symbol(address);
@@ -408,7 +408,7 @@ namespace emscripten {
         }
 
         template<typename... Args>
-        val operator()(Args&&... args) {
+        val operator()(Args&&... args) const {
             return internalCall(internal::_emval_call, std::forward<Args>(args)...);
         }
 
@@ -419,16 +419,17 @@ namespace emscripten {
             return MethodCaller<ReturnValue, Args...>::call(handle, name, std::forward<Args>(args)...);
         }
 
-        template<typename T>
-        T as() const {
+        template<typename T, typename ...Policies>
+        T as(Policies...) const {
             using namespace internal;
 
             typedef BindingType<T> BT;
+            typename WithPolicies<Policies...>::template ArgTypeList<T> targetType;
 
             EM_DESTRUCTORS destructors;
             EM_GENERIC_WIRE_TYPE result = _emval_as(
                 handle,
-                TypeID<T>::get(),
+                targetType.getTypes()[0],
                 &destructors);
             DestructorsRunner dr(destructors);
             return fromGenericWireType<T>(result);
@@ -460,7 +461,7 @@ namespace emscripten {
         }
 
         template<typename Implementation, typename... Args>
-        val internalCall(Implementation impl, Args&&... args)const {
+        val internalCall(Implementation impl, Args&&... args) const {
             using namespace internal;
 
             WithPolicies<>::ArgTypeList<Args...> argList;
